using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using FluentAssertions;
using Newtonsoft.Json;
using NUnit.Framework;

namespace E2ETests.Subscriber;

[Order(200)]
[TestFixture]
public class SubscriberTestHappyPath : SubscriberTestBase {
    [Order(201)]
    [Test]
    public void GetAvailableSubmissions_WithDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = subscriber.GetAvailableSubmissions(desitnationId).ToList();

        // Assert
        submissions.Count().Should().BeGreaterThanOrEqualTo(2);
        submissions.All(s => s.DestinationId == desitnationId).Should().BeTrue();
    }

    [Order(202)]
    [Test]
    public void GetAvailableSubmissions_WithOutDestinationId_ShouldReturnSubmissionsForPickupDto() {
        // Act
        var submissions = subscriber.GetAvailableSubmissions().ToList();

        // Assert
        submissions.Count().Should().BeGreaterThan(0);
    }


    [Order(1001)]
    [Test]
    public void GetAllSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto() {
        // Arrange
        var errorCounter = 0;
        var submissions = subscriber.GetAvailableSubmissions().ToList();
        submissions.Count().Should().BeGreaterThan(0);
        var i = 0;
        foreach (var submissionId in submissions.Select(s => s.SubmissionId)) {
            // var submissionId = "7c7bec6f-84c4-4ab5-9d3e-246b7881c8c3"; // submissions.Last().SubmissionId;

            // Act

            Console.WriteLine($"Getting submission {submissionId}");
            var dto = subscriber.RequestSubmission(submissionId);

            // Assert


            errorCounter.Should().BeLessThan(submissions.Count());
            Console.WriteLine($"Error counter: {errorCounter}/{submissions.Count()}");
            /*
             Console.WriteLine(dto.Submission.Data);
             Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
             */
            if (i++ % 2 == 0)
                dto.AcceptSubmission();
            else
                dto.RejectSubmission();
        }
    }

    [Ignore("ID is already been used")]
    [TestCase("303c60d3-3f6f-4913-bb51-4e66fe133c7f")]
    public void GetAttachment_WithSubmissionId_ShouldReturnSubmissionsForPickupDto(
        string submissionId) {
        // Act
        var attachments = subscriber.RequestSubmission(submissionId).GetAttachments();
        foreach (var attachment in attachments) {
            attachment.Content.Length.Should().BeGreaterThan(0);
            File.WriteAllBytes("attachments/test.pdf", attachment.Content);
        }
    }

    //[Ignore("Takes a while")]
    [Test]
    [Order(204)]
    public void GetAttachment_FromAllPendingSubmission_ShouldReturnAttachment() {
        // Arrange
        var submissions = subscriber.GetAvailableSubmissions().ToList();
        submissions.Count().Should().BeGreaterThan(0);

        foreach (var submission in submissions) {
            Console.WriteLine(
                $"Getting submission {submission.SubmissionId} - case {submission.CaseId}");
            var submissionId = submission.SubmissionId!;
            if (!Directory.Exists($"./attachments/{submissionId}/"))
                Directory.CreateDirectory($"./attachments/{submissionId}/");


            var subscriberWithSubmission = subscriber
                .RequestSubmission(submissionId, true);
            var attachments = subscriberWithSubmission
                .GetAttachments();
            foreach (var attachment in attachments)
                if ((attachment?.Content?.Length ?? 0) > 0)
                    File.WriteAllBytes(
                        Path.Combine($"./attachments/{submissionId}/",
                            attachment.Filename),
                        attachment.Content);


            // subscriberWithSubmission.AcceptSubmission();
            Console.WriteLine($"Success {submissionId}");
        }


        // Process.Start("open", "./attachments");
    }

    [Ignore("ID is already been used")]
    [TestCase("b0bd846f-f56a-4d06-a586-14fb0ea94fd0")]
    public void GetSingleSubmission_WithSubmissionId_ShouldReturnSubmissionsForPickupDto(
        string submissionId) {
        // Arrange
        var submissions = subscriber.GetAvailableSubmissions().ToList();
        submissions.Count().Should().BeGreaterThan(0);

        Console.WriteLine($"Getting submission {submissionId}");
        var dto = subscriber.RequestSubmission(submissionId);

        // Assert
        dto.Submission.Id.Should().Be(submissionId);


        Console.WriteLine(dto.Submission.Data);
        Console.WriteLine(JsonConvert.SerializeObject(dto.Submission.Metadata));
    }
}
